import argparse
import datetime
import json

from epevermodbus.driver import EpeverChargeController


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--portname", help="Port name for example /dev/ttyUSB0", default="/dev/ttyUSB0"
    )
    parser.add_argument(
        "--slaveaddress", help="Slave address 1-247", default=1, type=int
    )

    parser.add_argument(
        "--baudrate", help="Baudrate to communicate with controller (default is 115200)", default=115200, type=int
    )

    parser.add_argument("--json", help="Make a json output", action="store_true")
    parser.add_argument("--set-time", help="Set the RTC of the MPPT and exit", action="store_true")
    parser.add_argument("--set-battery-capacity", help="Set the battery capacity in Ah an exit", type=int)
    parser.add_argument(
        "--set-battery-temp-comp-coeff",
        help="Sets the batteries temperature compensation coefficient. Coefficient is in mV/°C/Cell without the sign",
        type=float,
    )
    parser.add_argument(
        "--set-over-voltage-disconnect-voltage",
        help="Set the over-voltage disconnect voltage and exit",
        type=float
    )
    parser.add_argument(
        "--set-over-voltage-reconnect-voltage",
        help="Set the over-voltage reconnect voltage and exit",
        type=float
    )
    parser.add_argument(
        "--set-charging-limit-voltage",
        help="Set the charging voltage limit and exit",
        type=float
    )
    parser.add_argument(
        "--set-discharging-limit-voltage",
        help="Set the discharging limit voltage and exit",
        type=float
    )
    parser.add_argument(
        "--set-boost-charging-voltage",
        help="Set the boost charging voltage and exit",
        type=float
    )
    parser.add_argument(
        "--set-boost-reconnect-charging-voltage",
        help="Set the boost reconnect charging voltage and exit",
        type=float
    )
    parser.add_argument(
        "--set-equalize-charging-voltage",
        help="Set the equalize charging voltage and exit",
        type=float
    )
    parser.add_argument(
        "--set-float-charging-voltage",
        help="Set the float charging voltage and exit",
        type=float
    )
    parser.add_argument(
        "--set-low-voltage-disconnect-voltage",
        help="Set the low-voltage disconnect voltage and exit",
        type=float
    )
    parser.add_argument(
        "--set-low-voltage-reconnect-voltage",
        help="Set the low voltage reconnect voltage and exit",
        type=float
    )
    parser.add_argument(
        "--set-under-voltage-warning-voltage",
        help="Set the under-voltage warning voltage and exit",
        type=float
    )
    parser.add_argument(
        "--set-under-voltage-recover-voltage",
        help="Set the under-voltage recover voltage and exit",
        type=float
    )
    args = parser.parse_args()

    controller = EpeverChargeController(args.portname, args.slaveaddress, args.baudrate)

    if args.set_time:
        print(f"Old RTC value: {controller.get_rtc()}")
        controller.set_rtc(datetime.datetime.now())
        print(f"New RTC value: {controller.get_rtc()}")

    if args.set_battery_capacity:
        print(f"Old capacity: {controller.get_battery_capacity()}AH")
        controller.set_battery_capacity(args.set_battery_capacity)
        print(f"New capacity: {controller.get_battery_capacity()}AH")

    if args.set_battery_temp_comp_coeff:
        print(
            "Old Temperature compensation coefficient: "
            f"{controller.get_temperature_compensation_coefficient()}mV/°C/Cell"
        )
        controller.set_temperature_compensation_coefficient(args.set_battery_temp_comp_coeff)
        print(
            "New Temperature compensation coefficient: "
            f"{controller.get_temperature_compensation_coefficient()}mV/°C/Cell"
        )

    if args.set_over_voltage_disconnect_voltage:
        print(f"Old over-voltage disconnect voltage: {controller.get_over_voltage_disconnect_voltage()}V")
        controller.set_battery_voltage_control_registers(
                over_voltage_disconnect_voltage=args.set_over_voltage_disconnect_voltage
        )
        print(f"New over-voltage disconnect voltage: {controller.get_over_voltage_disconnect_voltage()}V")

    if args.set_over_voltage_reconnect_voltage:
        print(f"Old over-voltage reconnect voltage: {controller.get_over_voltage_reconnect_voltage()}V")
        controller.set_battery_voltage_control_registers(
            over_voltage_reconnect_voltage=args.set_over_voltage_reconnect_voltage
        )
        print(f"New over-voltage reconnect voltage: {controller.get_over_voltage_reconnect_voltage()}V")

    if args.set_charging_limit_voltage:
        print(f"Old charging limit voltage: {controller.get_charging_limit_voltage()}V")
        controller.set_battery_voltage_control_registers(
            charging_limit_voltage=args.set_charging_limit_voltage
        )
        print(f"New charging limit voltage: {controller.get_charging_limit_voltage()}V")

    if args.set_discharging_limit_voltage:
        print(f"Old discharging limit voltage: {controller.get_discharging_limit_voltage()}V")
        controller.set_battery_voltage_control_registers(
            discharging_limit_voltage=args.set_discharging_limit_voltage
        )
        print(f"New discharging limit voltage: {controller.get_discharging_limit_voltage()}V")

    if args.set_boost_charging_voltage:
        print(f"Old boost charging voltage: {controller.get_boost_charging_voltage()}V")
        controller.set_battery_voltage_control_registers(boost_charging_voltage=args.set_boost_charging_voltage)
        print(f"New boost charging voltage: {controller.get_boost_charging_voltage()}V")

    if args.set_boost_reconnect_charging_voltage:
        print(f"Old boost reconnect charging voltage: {controller.get_boost_reconnect_charging_voltage()}V")
        controller.set_battery_voltage_control_registers(
            boost_reconnect_charging_voltage=args.set_boost_reconnect_charging_voltage
        )
        print(f"New boost reconnect charging voltage: {controller.get_boost_reconnect_charging_voltage()}V")

    if args.set_equalize_charging_voltage:
        print(f"Old equalize charging voltage: {controller.get_equalize_charging_voltage()}V")
        controller.set_battery_voltage_control_registers(
            equalize_charging_voltage=args.set_equalize_charging_voltage
        )
        print(f"New equalize charging voltage: {controller.get_equalize_charging_voltage()}V")

    if args.set_float_charging_voltage:
        print(f"Old float charging voltage: {controller.get_float_charging_voltage()}V")
        controller.set_battery_voltage_control_registers(float_charging_voltage=args.set_float_charging_voltage)
        print(f"New float charging voltage: {controller.get_float_charging_voltage()}V")

    if args.set_low_voltage_disconnect_voltage:
        print(f"Old low-voltage disconnect voltage: {controller.get_low_voltage_disconnect_voltage()}V")
        controller.set_battery_voltage_control_registers(
            low_voltage_disconnect_voltage=args.set_low_voltage_disconnect_voltage
        )
        print(f"New low-voltage disconnect voltage: {controller.get_low_voltage_disconnect_voltage()}V")

    if args.set_low_voltage_reconnect_voltage:
        print(f"Old low-voltage reconnect voltage: {controller.get_low_voltage_reconnect_voltage()}V")
        controller.set_battery_voltage_control_registers(
            low_voltage_reconnect_voltage=args.set_low_voltage_reconnect_voltage
        )
        print(f"New low-voltage reconnect voltage: {controller.get_low_voltage_reconnect_voltage()}V")

    if args.set_under_voltage_warning_voltage:
        print(f"Old under-voltage warning voltage: {controller.get_under_voltage_warning_voltage()}V")
        controller.set_battery_voltage_control_registers(
            under_voltage_warning_voltage=args.set_under_voltage_warning_voltage
        )
        print(f"New under-voltage warning voltage: {controller.get_under_voltage_warning_voltage()}V")

    if args.set_under_voltage_recover_voltage:
        print(f"Old under-voltage recover voltage: {controller.get_under_voltage_recover_voltage()}V")
        controller.set_battery_voltage_control_registers(
            under_voltage_recover_voltage=args.set_under_voltage_recover_voltage
        )
        print(f"New under-voltage recover voltage: {controller.get_under_voltage_recover_voltage()}V")

    if any([
        args.set_time,
        args.set_battery_capacity,
        args.set_battery_temp_comp_coeff,
        args.set_over_voltage_disconnect_voltage,
        args.set_over_voltage_reconnect_voltage,
        args.set_charging_limit_voltage,
        args.set_discharging_limit_voltage,
        args.set_boost_charging_voltage,
        args.set_boost_reconnect_charging_voltage,
        args.set_equalize_charging_voltage,
        args.set_float_charging_voltage,
        args.set_low_voltage_disconnect_voltage,
        args.set_low_voltage_reconnect_voltage,
        args.set_under_voltage_warning_voltage,
        args.set_under_voltage_recover_voltage,
    ]):
        exit(0)

    if args.json:
        output = {}
        output["solar_voltage"] = controller.get_solar_voltage()
        output["solar_current"] = controller.get_solar_current()
        output["solar_power"] = controller.get_solar_power()
        output["load_voltage"] = controller.get_load_voltage()
        output["load_current"] = controller.get_load_current()
        output["load_power"] = controller.get_load_power()
        output["battery_voltage"] = controller.get_battery_voltage()
        output["battery_current"] = controller.get_battery_current()
        output["battery_power"] = controller.get_battery_power()
        output["battery_state_of_charge"] = controller.get_battery_state_of_charge()
        output["battery_temperature"] = controller.get_battery_temperature()
        output["remote_battery_temperature"] = controller.get_remote_battery_temperature()
        output["controller_temperature"] = controller.get_controller_temperature()
        output["battery_status"] = controller.get_battery_status()
        output["charging_equipment_status"] = controller.get_charging_equipment_status()
        output["discharging_equipment_status"] = controller.get_discharging_equipment_status()
        output["day_time"] = controller.is_day()
        output["night_time"] = controller.is_night()
        output["maximum_battery_voltage_today"] = controller.get_maximum_battery_voltage_today()
        output["minimum_battery_voltage_today"] = controller.get_minimum_battery_voltage_today()
        output["maximum_pv_voltage_today"] = controller.get_maximum_pv_voltage_today()
        output["minimum_pv_voltage_today"] = controller.get_minimum_pv_voltage_today()
        output["device_over_temperature"] = controller.is_device_over_temperature()
        output["consumed_energy_today"] = controller.get_consumed_energy_today()
        output["consumed_energy_this_month"] = controller.get_consumed_energy_this_month()
        output["consumed_energy_this_year"] = controller.get_consumed_energy_this_year()
        output["total_consumed_energy"] = controller.get_total_consumed_energy()
        output["generated_energy_today"] = controller.get_generated_energy_today()
        output["generated_energy_this_month"] = controller.get_generated_energy_this_month()
        output["generated_energy_this_year"] = controller.get_generated_energy_this_year()
        output["total_generated_energy"] = controller.get_total_generated_energy()
        output["current_device_time"] = controller.get_rtc().isoformat()
        output["rated_charging_current"] = controller.get_rated_charging_current()
        output["rated_load_current"] = controller.get_rated_load_current()
        output["battery_real_rated_voltage"] = controller.get_battery_real_rated_voltage()
        output["battery_type"] = controller.get_battery_type()
        output["battery_capacity"] = controller.get_battery_capacity()
        output["temperature_compensation_coefficient"] = controller.get_temperature_compensation_coefficient()
        output["over_voltage_disconnect_voltage"] = controller.get_over_voltage_disconnect_voltage()
        output["charging_limit_voltage"] = controller.get_charging_limit_voltage()
        output["over_voltage_reconnect_voltage"] = controller.get_over_voltage_reconnect_voltage()
        output["equalize_charging_voltage"] = controller.get_equalize_charging_voltage()
        output["boost_charging_voltage"] = controller.get_boost_charging_voltage()
        output["float_charging_voltage"] = controller.get_float_charging_voltage()
        output["boost_reconnect_charging_voltage"] = controller.get_boost_reconnect_charging_voltage()
        output["low_voltage_reconnect_voltage"] = controller.get_low_voltage_reconnect_voltage()
        output["under_voltage_recover_voltage"] = controller.get_under_voltage_recover_voltage()
        output["under_voltage_warning_voltage"] = controller.get_under_voltage_warning_voltage()
        output["low_voltage_disconnect_voltage"] = controller.get_low_voltage_disconnect_voltage()
        output["discharging_limit_voltage"] = controller.get_discharging_limit_voltage()
        output["battery_rated_voltage"] = controller.get_battery_rated_voltage()
        output["default_load_on_off_in_manual_mode"] = controller.get_default_load_on_off_in_manual_mode()
        output["equalize_duration"] = controller.get_equalize_duration()
        output["boost_duration"] = controller.get_boost_duration()
        output["battery_discharge"] = controller.get_battery_discharge()
        output["battery_charge"] = controller.get_battery_charge()
        output["charging_mode"] = controller.get_charging_mode()
        print(json.dumps(output))
    else:
        print("Real Time Data")
        print(f"Solar voltage: {controller.get_solar_voltage()}V")
        print(f"Solar current: {controller.get_solar_current()}A")
        print(f"Solar power: {controller.get_solar_power()}W")
        print(f"Load voltage: {controller.get_load_voltage()}V")
        print(f"Load current: {controller.get_load_current()}A")
        print(f"Load power: {controller.get_load_power()}W")
        print(f"Battery voltage: {controller.get_battery_voltage()}V")
        print(f"Battery current: {controller.get_battery_current()}A")
        print(f"Battery power: {controller.get_battery_power()}W")
        print(
            f"Battery state of charge: {controller.get_battery_state_of_charge()}%")
        print(f"Battery temperature: {controller.get_battery_temperature()}°C")
        print(
            f"Remote battery temperature: {controller.get_remote_battery_temperature()}°C"
        )
        print(
            f"Controller temperature: {controller.get_controller_temperature()}°C")
        print(f"Battery status: {controller.get_battery_status()}")
        print(
            f"Charging equipment status: {controller.get_charging_equipment_status()}")
        print(
            f"Discharging equipment status: {controller.get_discharging_equipment_status()}"
        )
        print(f"Day time: {controller.is_day()}")
        print(f"Night time: {controller.is_night()}")
        print(
            f"Maximum battery voltage today: {controller.get_maximum_battery_voltage_today()}V"
        )
        print(
            f"Minimum battery voltage today: {controller.get_minimum_battery_voltage_today()}V"
        )
        print(f"Maximum PV voltage today: {controller.get_maximum_pv_voltage_today()}V")
        print(f"Minimum PV voltage today: {controller.get_minimum_pv_voltage_today()}V")
        print(
            f"Device over temperature: {controller.is_device_over_temperature()}")
        print(f"Consumed energy today: {controller.get_consumed_energy_today()}kWh")
        print(f"Consumed energy this month: {controller.get_consumed_energy_this_month()}kWh")
        print(f"Consumed energy this year: {controller.get_consumed_energy_this_year()}kWh")
        print(f"Total consumed energy: {controller.get_total_consumed_energy()}kWh")
        print(f"Generated energy today: {controller.get_generated_energy_today()}kWh")
        print(f"Generated energy this month: {controller.get_generated_energy_this_month()}kWh")
        print(f"Generated energy this year: {controller.get_generated_energy_this_year()}kWh")
        print(f"Total generated energy: {controller.get_total_generated_energy()}kWh")
        print(f"Current device time: {controller.get_rtc()}")
        print("\n")
    
        print("Battery Parameters:")
        print(
            f"Rated charging current: {controller.get_rated_charging_current()}A")
        print(f"Rated load current: {controller.get_rated_load_current()}A")
        print(
            f"Battery real rated voltage: {controller.get_battery_real_rated_voltage()}V")
        print(f"Battery type: {controller.get_battery_type()}")
        print(f"Battery capacity: {controller.get_battery_capacity()}AH")
        print(
            "Temperature compensation coefficient: "
            f"{controller.get_temperature_compensation_coefficient()}mV/°C/Cell"
        )
        print(
            f"Over voltage disconnect voltage: {controller.get_over_voltage_disconnect_voltage()}V"
        )
        print(
            f"Charging limit voltage: {controller.get_charging_limit_voltage()}V")
        print(
            f"Over voltage reconnect voltage: {controller.get_over_voltage_reconnect_voltage()}V"
        )
        print(
            f"Equalize charging voltage: {controller.get_equalize_charging_voltage()}V")
        print(
            f"Boost charging voltage: {controller.get_boost_charging_voltage()}V")
        print(
            f"Float charging voltage: {controller.get_float_charging_voltage()}V")
        print(
            f"Boost reconnect charging voltage: {controller.get_boost_reconnect_charging_voltage()}V"
        ),
        print(
            f"Low voltage reconnect voltage: {controller.get_low_voltage_reconnect_voltage()}V"
        )
        print(
            f"Under voltage recover voltage: {controller.get_under_voltage_recover_voltage()}V"
        )
        print(
            f"Under voltage warning voltage: {controller.get_under_voltage_warning_voltage()}V"
        )
        print(
            f"Low voltage disconnect voltage: {controller.get_low_voltage_disconnect_voltage()}V"
        )
        print(
            f"Discharging limit voltage: {controller.get_discharging_limit_voltage()}V")
        print(f"Battery rated voltage: {controller.get_battery_rated_voltage()}")
        print(
            "Default load on/off in manual mode:",
            controller.get_default_load_on_off_in_manual_mode(),
        )
        print(f"Equalize duration: {controller.get_equalize_duration()} min")
        print(f"Boost duration: {controller.get_boost_duration()} min")
        print(f"Battery discharge: {controller.get_battery_discharge()}%")
        print(f"Battery charge: {controller.get_battery_charge()}%")
        print(f"Charging mode: {controller.get_charging_mode()}")


if __name__ == "__main__":
    main()



import datetime

import minimalmodbus
import serial
from retrying import retry

from epevermodbus.extract_bits import extract_bits


class EpeverChargeController(minimalmodbus.Instrument):
    """Instrument class for Epever Charge Controllers.

    Args:
        * portname (str): port name
        * slaveaddress (int): slave address in the range 1 to 247
        * baudrate (int): baudrate to communicate with controller (default is 115200)

    """

    battery_voltage_control_register_names = [
        "over_voltage_disconnect_voltage",
        "charging_limit_voltage",
        "over_voltage_reconnect_voltage",
        "equalize_charging_voltage",
        "boost_charging_voltage",
        "float_charging_voltage",
        "boost_reconnect_charging_voltage",
        "low_voltage_reconnect_voltage",
        "under_voltage_recover_voltage",
        "under_voltage_warning_voltage",
        "low_voltage_disconnect_voltage",
        "discharging_limit_voltage"
    ]

    def __init__(self, portname, slaveaddress, baudrate=115200):
        minimalmodbus.Instrument.__init__(self, portname, slaveaddress)
        self.serial.baudrate = baudrate
        self.serial.bytesize = 8
        self.serial.parity = serial.PARITY_NONE
        self.serial.stopbits = 1
        self.serial.timeout = 1
        self.mode = minimalmodbus.MODE_RTU
        self.clear_buffers_before_each_transaction = True

    @retry(wait_fixed=200, stop_max_attempt_number=5)
    def retriable_read_register(
        self, registeraddress, number_of_decimals, functioncode, signed=False
    ):
        return self.read_register(
            registeraddress, number_of_decimals, functioncode, signed
        )

    @retry(wait_fixed=200, stop_max_attempt_number=5)
    def retriable_read_registers(
        self, registeraddress, number_of_registers, functioncode
    ):
        return self.read_registers(
            registeraddress, number_of_registers, functioncode
        )

    @retry(wait_fixed=200, stop_max_attempt_number=5)
    def retriable_read_long(
        self, registeraddress, functioncode, signed=False, byteorder=minimalmodbus.BYTEORDER_LITTLE_SWAP
    ):
        return self.read_long(
            registeraddress, functioncode, signed, byteorder
        )

    @retry(wait_fixed=200, stop_max_attempt_number=5)
    def retriable_read_bit(self, registeraddress, functioncode):
        return self.read_bit(registeraddress, functioncode)

    def get_solar_voltage(self):
        """PV array input in volts"""
        return self.retriable_read_register(0x3100, 2, 4)

    def get_solar_current(self):
        """PV array input in amps"""
        return self.retriable_read_register(0x3101, 2, 4)

    def get_solar_power(self):
        """PV array input power"""
        return self.retriable_read_long(0x3102, 4) / 100

    def get_load_voltage(self):
        """Load output in volts"""
        return self.retriable_read_register(0x310C, 2, 4)

    def get_load_current(self):
        """Load output in amps"""
        return self.retriable_read_register(0x310D, 2, 4)

    def get_load_power(self):
        """Load output in watts"""
        return self.retriable_read_long(0x310E, 4) / 100

    def get_battery_current(self):
        """Battery current in amps"""
        return self.retriable_read_long(0x331B, 4, signed=True) / 100

    def get_battery_voltage(self):
        """Battery voltage"""
        return self.retriable_read_register(0x331A, 2, 4)

    def get_battery_power(self):
        """Battery power in watts"""
        return self.retriable_read_long(0x3106, 4) / 100

    def get_battery_state_of_charge(self):
        """Battery state of charge"""
        return self.retriable_read_register(0x311A, 0, 4)

    def get_battery_temperature(self):
        """battery temperature"""
        return self.retriable_read_register(0x3110, 2, 4, signed=True)

    def get_remote_battery_temperature(self):
        """The battery temperature measured by remote temperature sensor"""
        return self.retriable_read_register(0x311B, 2, 4, signed=True)

    def get_controller_temperature(self):
        """Temperature inside equipment"""
        return self.retriable_read_register(0x3111, 2, 4, signed=True)

    def get_battery_status(self):
        """Battery status"""
        register_value = self.retriable_read_register(0x3200, 0, 4)

        # D7-4
        temperature_warning_status = {
            0: "NORMAL",
            1: "OVER_TEMP",  # Higher than warning settings
            2: "LOW_TEMP",  # Lower than warning settings
        }[extract_bits(register_value, 4, 0b111)]

        # D3-0
        battery_status = {
            0: "NORMAL",
            1: "OVER_VOLTAGE",
            2: "UNDER_VOLTAGE",
            3: "OVER_DISCHARGE",
            4: "FAULT",
        }[extract_bits(register_value, 0, 0b111)]

        return {
            "wrong_identifaction_for_rated_voltage": bool(
                extract_bits(register_value, 15, 0b1)
            ),
            "battery_inner_resistence_abnormal": bool(
                extract_bits(register_value, 8, 0b1)
            ),
            "temperature_warning_status": temperature_warning_status,
            "battery_status": battery_status,
        }

    def get_charging_equipment_status(self):
        """Charging equipment status"""
        register_value = self.retriable_read_register(0x3201, 0, 4)

        # D15-14
        input_voltage_status = {
            0: "NORMAL",
            1: "NO_INPUT_POWER",
            2: "HIGHER_INPUT",
            3: "INPUT_VOLTAGE_ERROR",
        }[extract_bits(register_value, 14, 0b11)]

        # D3-2
        charging_status = {
            0: "NO_CHARGING",
            1: "FLOAT",
            2: "BOOST",
            3: "EQUALIZATION",
        }[extract_bits(register_value, 2, 0b11)]

        return {
            "input_voltage_status": input_voltage_status,
            "charging_mosfet_is_short_circuit": bool(
                extract_bits(register_value, 13, 0b1)
            ),
            "charging_or_anti_reverse_mosfet_is_open_circuit": bool(
                extract_bits(register_value, 12, 0b1)
            ),
            "anti_reverse_mosfet_is_short_circuit": bool(
                extract_bits(register_value, 11, 0b1)
            ),
            "input_over_current": bool(extract_bits(register_value, 10, 0b1)),
            "load_over_current": bool(extract_bits(register_value, 9, 0b1)),
            "load_short_circuit": bool(extract_bits(register_value, 8, 0b1)),
            "load_mosfet_short_circuit": bool(extract_bits(register_value, 7, 0b1)),
            "disequilibrium_in_three_circuits": bool(
                extract_bits(register_value, 6, 0b1)
            ),
            "pv_input_short_circuit": bool(extract_bits(register_value, 4, 0b1)),
            "charging_status": charging_status,
            "fault": bool(
                extract_bits(register_value, 1, 0b1)
            ),  # this does not seem to be functioning correctly. Fault status is returned when no fault.
            "running": bool(extract_bits(register_value, 0, 0b1)),
        }

    def get_discharging_equipment_status(self):
        """Charging equipment status"""
        register_value = self.retriable_read_register(0x3202, 0, 4)

        # D15-14
        input_voltage_status = {
            0: "NORMAL",
            1: "LOW",
            2: "HIGH",
            3: "NO_ACCESS",
        }[extract_bits(register_value, 14, 0b11)]

        # D13-12
        output_power_load = {
            0: "LIGHT",
            1: "MODERATE",
            2: "RATED",
            3: "OVERLOAD",
        }[extract_bits(register_value, 12, 0b11)]

        return {
            "input_voltage_status": input_voltage_status,
            "output_power_load": output_power_load,
            "short_circuit": bool(extract_bits(register_value, 11, 0b1)),
            "unable_to_discharge": bool(extract_bits(register_value, 10, 0b1)),
            "unable_to_stop_discharging": bool(extract_bits(register_value, 9, 0b1)),
            "output_voltage_abnormal": bool(extract_bits(register_value, 8, 0b1)),
            "input_over_voltage": bool(extract_bits(register_value, 7, 0b1)),
            "short_circuit_in_high_voltage_side": bool(
                extract_bits(register_value, 6, 0b1)
            ),
            "boost_over_voltage": bool(extract_bits(register_value, 5, 0b1)),
            "output_over_voltage": bool(extract_bits(register_value, 4, 0b1)),
            "output_over_voltage": bool(extract_bits(register_value, 4, 0b1)),
            "fault": bool(extract_bits(register_value, 1, 0b1)),
            "running": bool(extract_bits(register_value, 0, 0b1)),
        }

    def is_day(self):
        """Is day time"""
        return not self.is_night()

    def is_night(self):
        """Is night time"""
        return True if self.retriable_read_bit(0x200C, 2) == 1 else False

    def is_device_over_temperature(self):
        """Over temperature inside the device"""
        return True if self.retriable_read_bit(0x2000, 2) == 1 else False

    def get_maximum_battery_voltage_today(self):
        """Maximum battery voltage today"""
        return self.retriable_read_register(0x3302, 2, 4)

    def get_minimum_battery_voltage_today(self):
        """Minimum battery voltage today"""
        return self.retriable_read_register(0x3303, 2, 4)

    def get_rated_charging_current(self):
        """Rated charging current"""
        return self.retriable_read_register(0x3005, 2, 4)

    def get_rated_load_current(self):
        """Rated load current"""
        return self.retriable_read_register(0x300E, 2, 4)

    def get_battery_real_rated_voltage(self):
        """Battery real rated voltage"""
        return self.retriable_read_register(0x311D, 2, 4)

    def get_battery_type(self):
        """Battery type"""
        return {
            0: "USER_DEFINED",
            1: "SEALED",
            2: "GEL",
            3: "FLOODED",
            4: "LIFEPO4",
            5: "LIFEPO4",
            6: "LIFEPO4",
            7: "LIFEPO4",
            8: "LI_NICOMN_O2",
            9: "LI_NICOMN_O2",
            10: "LI_NICOMN_O2",
            11: "LI_NICOMN_O2",
            12: "LI_NICOMN_O2",
        }[self.retriable_read_register(0x9000, 0, 3)]

    def get_battery_capacity(self):
        """Battery capacity in amp hours"""
        return self.retriable_read_register(0x9001, 0, 3)

    def set_battery_capacity(self, capacity: int):
        """Set Battery capacity in amp hours"""
        return self.write_register(0x9001, capacity)

    def get_temperature_compensation_coefficient(self):
        """Temperature compensation coefficient"""
        return self.retriable_read_register(0x9002, 2, 3)

    def set_temperature_compensation_coefficient(self, coefficient: float):
        """Set the Temperature compensation coefficient"""
        return self.write_register(0x9002, coefficient*100)

    def get_battery_voltage_control_registers(self):
        """Returns all 12 battery voltage control settings"""
        register_values = self.retriable_read_registers(0x9003, 12, 3)
        return {
            register_name: register_values[idx] / 100
            for idx, register_name in enumerate(self.battery_voltage_control_register_names)
        }

    def set_battery_voltage_control_registers(self, **kwargs):
        """Sets from 1 to 12 battery voltage control settings

        Args:
        * keyword arguments (float)

        The provided arguments must:
        * have names in battery_voltage_control_register_names
        * be one or more in number
        """
        self.set_battery_voltage_control_registers_dict(kwargs)

    def set_battery_voltage_control_registers_dict(self, control_registers: dict):
        """Sets from 1 to 12 battery voltage control settings

        Args:
        * control_registers (dict)

        The provided dict must:
        * have key names in battery_voltage_control_register_names
        * have one or more key names.
        """
        if not len(control_registers):
            raise TypeError(
                "set_battery_voltage_control_registers() missing keyword arguments"
            )

        if not all([
            kw_key in self.battery_voltage_control_register_names
            for kw_key in control_registers.keys()
        ]):
            raise TypeError(
                "set_battery_voltage_control_registers() got an unexpected keyword argument"
            )

        values_dict = self.get_battery_voltage_control_registers()
        values_dict.update(control_registers)

        values = [
            int(values_dict[register_name] * 100)
            for register_name in self.battery_voltage_control_register_names
        ]

        self.write_registers(0x9003, values)
        return

    def get_over_voltage_disconnect_voltage(self):
        """Over voltage disconnect voltage"""
        return self.retriable_read_register(0x9003, 2, 3)

    def get_charging_limit_voltage(self):
        """Charging limit voltage"""
        return self.retriable_read_register(0x9004, 2, 3)

    def get_over_voltage_reconnect_voltage(self):
        """Over voltage reconnect voltage"""
        return self.retriable_read_register(0x9005, 2, 3)

    def get_equalize_charging_voltage(self):
        """Equalize charging voltage"""
        return self.retriable_read_register(0x9006, 2, 3)

    def get_boost_charging_voltage(self):
        """Boost charging voltage"""
        return self.retriable_read_register(0x9007, 2, 3)

    def get_float_charging_voltage(self):
        """Float charging voltage"""
        return self.retriable_read_register(0x9008, 2, 3)

    def get_boost_reconnect_charging_voltage(self):
        """Boost reconnect charging voltage"""
        return self.retriable_read_register(0x9009, 2, 3)

    def get_low_voltage_reconnect_voltage(self):
        """Low voltage reconnect voltage"""
        return self.retriable_read_register(0x900A, 2, 3)

    def get_under_voltage_recover_voltage(self):
        """Under voltage warning recover voltage"""
        return self.retriable_read_register(0x900B, 2, 3)

    def get_under_voltage_warning_voltage(self):
        """Under voltage warning voltage"""
        return self.retriable_read_register(0x900C, 2, 3)

    def get_low_voltage_disconnect_voltage(self):
        """Low voltage disconnect voltage"""
        return self.retriable_read_register(0x900D, 2, 3)

    def get_discharging_limit_voltage(self):
        """Discharging limit voltage"""
        return self.retriable_read_register(0x900E, 2, 3)

    def get_battery_rated_voltage(self):
        """Battery rated voltage"""
        return {
            0: "AUTO",
            1: "12V",
            2: "24V",
            3: "36V",
            4: "48V",
            5: "60V",
            6: "110V",
            7: "120V",
            8: "220V",
            9: "240V",
        }[self.retriable_read_register(0x9067, 0, 3)]

    def get_default_load_on_off_in_manual_mode(self):
        """Default load On/Off in manual mode"""
        return {0: "OFF", 1: "ON"}[self.retriable_read_register(0x906A, 0, 3)]

    def get_equalize_duration(self):
        """Equalize duration"""
        return self.retriable_read_register(0x906B, 0, 3)

    def get_boost_duration(self):
        """Equalize duration"""
        return self.retriable_read_register(0x906C, 0, 3)

    def get_battery_discharge(self):
        """Battery discharge"""
        return self.retriable_read_register(0x906D, 0, 3)

    def get_battery_charge(self):
        """Battery charge"""
        return self.retriable_read_register(0x906E, 0, 3)

    def get_charging_mode(self):
        """Charging mode"""
        return {0: "VOLTAGE_COMPENSATION", 1: "SOC"}[
            self.retriable_read_register(0x9070, 0, 3)
        ]

    def get_total_consumed_energy(self):
        """Total consumed energy"""
        return self.retriable_read_long(0x330A, 4) / 100

    def get_total_generated_energy(self):
        """Total generated energy"""
        return self.retriable_read_long(0x3312, 4) / 100

    def get_maximum_pv_voltage_today(self):
        """Maximum PV voltage today"""
        return self.retriable_read_register(0x3300, 2, 4)

    def get_minimum_pv_voltage_today(self):
        """Minimum PV voltage today"""
        return self.retriable_read_register(0x3301, 2, 4)

    def get_consumed_energy_today(self):
        """Consumed energy today"""
        return self.retriable_read_long(0x3304, 4) / 100

    def get_consumed_energy_this_month(self):
        """Consumed energy this month"""
        return self.retriable_read_long(0x3306, 4) / 100

    def get_consumed_energy_this_year(self):
        """Consumed energy this year"""
        return self.retriable_read_long(0x3308, 4) / 100

    def get_generated_energy_today(self):
        """Generated energy today"""
        return self.retriable_read_long(0x330C, 4) / 100

    def get_generated_energy_this_month(self):
        """Generated energy this month"""
        return self.retriable_read_long(0x330E, 4) / 100

    def get_generated_energy_this_year(self):
        """Generated energy this year"""
        return self.retriable_read_long(0x3310, 4) / 100

    def get_rtc(self):
        """
        Reads the RTC.
        :return: datetime.datetime if successful
                 None otherwise
        """

        reg_ms = self.retriable_read_register(0x9013, 0, 3)
        second = extract_bits(reg_ms, 0, 0b1111_1111)
        minute = extract_bits(reg_ms, 8, 0b1111_1111)

        reg_hd = self.retriable_read_register(0x9014, 0, 3)
        hour = extract_bits(reg_hd, 0, 0b1111_1111)
        day = extract_bits(reg_hd, 8, 0b1111_1111)

        reg_my = self.retriable_read_register(0x9015, 0, 3)
        month = extract_bits(reg_my, 0, 0b1111_1111)
        year = extract_bits(reg_my, 8, 0b1111_1111)+2000

        try:
            dt = datetime.datetime(year, month, day, hour, minute, second)
        except ValueError:
            dt = None
        return dt

    def set_rtc(self, new_time: datetime.datetime):
        """
        Set the RTC
        :param new_time: The new value to set.
        :return: None
        """

        reg_ms = new_time.second + (new_time.minute << 8)
        reg_hd = new_time.hour + (new_time.day << 8)
        reg_my = new_time.month + ((new_time.year-2000) << 8)

        self.write_registers(0x9013, [reg_ms, reg_hd, reg_my])



from setuptools import setup

setup(
    name="epevermodbus",
    version="0.0.18",
    description="",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/rosswarren/epevermodbus",
    author="Ross Warren",
    author_email="rosswarren4@gmail.com",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
    ],
    packages=["epevermodbus"],
    include_package_data=True,
    install_requires=["minimalmodbus", "retrying"],
    test_suite="test",
    entry_points={
        "console_scripts": [
            "epevermodbus = epevermodbus.command_line:main",
        ],
    },
)
